package main

import (
	"context"
	"github.com/robfig/cron/v3"
	"project_management/app"
	"project_management/app/model"
	"project_management/database"
	"project_management/global"
	"project_management/nats"
	"project_management/simple_zap"
)

func init() {

	err := global.SetupSetting()
	if err != nil {
		simple_zap.WithCtx(context.Background()).Sugar().Warn(err, "配置文件读取失败")
	}
	err = global.SetupDBLink()
	if err != nil {
		simple_zap.WithCtx(context.Background()).Sugar().Warn(err, "数据库连接失败")
	}
	// 数据库迁移
	database.Migrate(global.DBLink)
	//nats
	nats.SetupNats()
	//redis
	global.SetupRedisLink()
	c := cron.New()
	_, err = c.AddFunc("0 0 0 * * ?", func() {
		check := model.Apply{}.Check()
		if !check {
			simple_zap.WithCtx(context.Background()).Sugar().Warn("项目初始化失败")
			return
		}
	})
	if err != nil {
		simple_zap.WithCtx(context.Background()).Sugar().Warn(err, "项目初始化失败")
		return
	}
	c.Start()
	select {}
}

// @title 项目管理
// @version 1.0
// @description 项目管理
// @Host 127.0.0.1:9999
// @BasePath /api
//
//go:generate swag init --parseDependency --parseDepth=6
func main() {
	err := app.InitRouter()

	if err != nil {
		simple_zap.WithCtx(context.Background()).Sugar().Warn(err, "项目初始化失败")
		return
	}

}
